<?php
/**
 * License Module
 *
 * @package blesta
 * @subpackage blesta.components.modules.license_module
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseModule extends Module {

	/**
	 * Initializes the module
	 */
	public function __construct() {
		// Load components required by this module
		Loader::loadComponents($this, array("Input"));

		// Load the language required by this module
		Language::loadLang("license_module", null, dirname(__FILE__) . DS . "language" . DS);

        // Load module config
        $this->loadConfig(dirname(__FILE__) . DS . "config.json");
	}

    /**
     * Performs migration of data from $current_version (the current installed version)
     * to the given file set version. Sets Input errors on failure, preventing
     * the module from being upgraded.
     *
     * @param string $current_version The current installed version of this module
     */
    public function upgrade($current_version)
    {
        if (version_compare($current_version, '1.7.0', '<')) {
            if (!isset($this->ModuleManager)) {
                Loader::loadModels($this, ['ModuleManager']);
            }

            if (version_compare(BLESTA_VERSION, "4.6.0", ">=")) {
                // Update all module rows to change support_package to support_packages
                $modules = $this->ModuleManager->getByClass('license_module');
                foreach ($modules as $module) {
                    $rows = $this->ModuleManager->getRows($module->id);
                    foreach ($rows as $row) {
                        $meta = (array)$row->meta;
                        $meta['support_packages'] = isset($row->meta->support_package)
                            ? [$row->meta->support_package]
                            : [];
                        unset($meta['support_package']);

                        $this->ModuleManager->editRow($row->id, $meta);
                    }
                }
            } else {
                $this->Input->setErrors(
                    [
                        'blesta_version' => [
                            'incompatible' => Language::_('LicenseModule.!error.blesta_version.incompatible.4_6', true)
                        ]
                    ]
                );
            }
        }
    }

	/**
	 * Adds the service to the remote server. Sets Input errors on failure,
	 * preventing the service from being added.
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being added (if the current service is an addon service service and parent service has already been provisioned)
	 * @param string $status The status of the service being added. These include:
	 * 	- active
	 * 	- canceled
	 * 	- pending
	 * 	- suspended
	 * @return array A numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function addService($package, array $vars=null, $parent_package=null, $parent_service=null, $status="pending") {
		$row = $this->getModuleRow();
		$license_key = null;
		$pool = $row->meta->key_pool;

		if ($status == "active") {
			if ($row->meta->key_method == "auto") {
				$exists = true;
				for ($i = 0; $i < 100 && $exists; $i++) {

					$key = null;
					for ($j = 0; $j < $row->meta->key_length; $j++)
						$key .= $pool[mt_rand(0, strlen($pool)-1)];

					$license_key = str_replace("{key}", $key, $row->meta->key_format);

					$exists = $this->getLicense($row->module_id, $license_key);
				}
				if ($exists) {
					$this->Input->setErrors(array(
						'license_module_key' => Language::_('LicenseModule.!error.license_module_key.not_unique', true)
					));
					return;
				}
			}
			else {
				#
				# TODO: Handle a list of license keys
				#
			}
		}

		return array(
			array(
				'key' => "license_module_key",
				'value' => $license_key,
			),
			array(
				'key' => "license_module_domains",
				'value' => array(),
			),
			array(
				'key' => "license_module_ips",
				'value' => array(),
			),
			array(
				'key' => "license_module_paths",
				'value' => array(),
			),
			array(
				'key' => "license_module_status",
				'value' => "reissue",
			),
			array(
				'key' => "license_module_callhome",
				'value' => "",
			),
			array(
				'key' => "license_module_version",
				'value' => "",
			)
		);
	}

	/**
	 * Edits the service on the remote server. Sets Input errors on failure,
	 * preventing the service from being edited.
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being edited (if the current service is an addon service)
	 * @return array A numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function editService($package, $service, array $vars=array(), $parent_package=null, $parent_service=null) {
		$row = $this->getModuleRow();

		// Ensure no other service has this license key
		$rules = array(
			'license_module_key' => array(
				'valid' => array(
					'if_set' => true,
					'rule' => array(array($this, "validateLicenseKeyUnique"), $row->module_id, $service->id),
					'message' => Language::_('LicenseModule.!error.license_module_key.valid', true)
				)
			)
		);

		$this->Input->setRules($rules);

		if ($this->Input->validates($vars)) {

			$service_fields = $this->serviceFieldsToObject($service->fields);

			if (!isset($vars['license_module_status']))
				$vars['license_module_status'] = "";

			// Update fields
			$service_fields->license_module_status = $vars['license_module_status'];
			if (isset($vars['license_module_key']))
				$service_fields->license_module_key = $vars['license_module_key'];

			return array(
				array(
					'key' => "license_module_key",
					'value' => $service_fields->license_module_key,
				),
				array(
					'key' => "license_module_domains",
					'value' => $service_fields->license_module_domains,
				),
				array(
					'key' => "license_module_ips",
					'value' => $service_fields->license_module_ips,
				),
				array(
					'key' => "license_module_paths",
					'value' => $service_fields->license_module_paths,
				),
				array(
					'key' => "license_module_status",
					'value' => $service_fields->license_module_status,
				),
				array(
					'key' => "license_module_callhome",
					'value' => $service_fields->license_module_callhome,
				),
				array(
					'key' => "license_module_version",
					'value' => isset($service_fields->license_module_version) ? $service_fields->license_module_version : "",
				)
			);
		}
	}

	/**
	 * Returns the rendered view of the manage module page
	 *
	 * @param mixed $module A stdClass object representing the module and its rows
	 * @param array $vars An array of post data submitted to or on the manager module page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the manager module page
	 */
	public function manageModule($module, array &$vars) {
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("manage", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);

		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));

		$this->view->set("module", $module);

		return $this->view->fetch();
	}

	/**
	 * Returns the rendered view of the add module row page
	 *
	 * @param array $vars An array of post data submitted to or on the add module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the add module row page
	 */
	public function manageAddRow(array &$vars) {
		Loader::loadModels($this, array("Packages"));

		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("add_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);

		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));

		$addon_packages = $this->Form->collapseObjectArray($this->Packages->getAll(Configure::get("Blesta.company_id"), array('name'=>"ASC"), null, "addon"), "name", "id");

		$this->view->set("vars", (object)$vars);
		$this->view->set("periods", $this->Packages->getPricingPeriods());
		$this->view->set("addon_packages", $addon_packages);
		$this->view->set("license_methods", $this->licenseMethods());
		$this->view->set("license_key_methods", $this->licenseKeyMethods());
		return $this->view->fetch();
	}

	/**
	 * Returns the rendered view of the edit module row page
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 * @param array $vars An array of post data submitted to or on the edit module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the edit module row page
	 */
	public function manageEditRow($module_row, array &$vars) {
		Loader::loadModels($this, array("Packages"));

		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("edit_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);

		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));

		if (empty($vars))
			$vars = $module_row->meta;

		$addon_packages = $this->Form->collapseObjectArray($this->Packages->getAll(Configure::get("Blesta.company_id"), array('name'=>"ASC"), null, "addon"), "name", "id");

		$this->view->set("vars", (object)$vars);
		$this->view->set("periods", $this->Packages->getPricingPeriods());
		$this->view->set("addon_packages", $addon_packages);
		$this->view->set("license_methods", $this->licenseMethods());
		$this->view->set("license_key_methods", $this->licenseKeyMethods());
		return $this->view->fetch();
	}

	/**
	 * Adds the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being added.
	 *
	 * @param array $vars An array of module info to add
	 * @return array A numerically indexed array of meta fields for the module row containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 */
	public function addModuleRow(array &$vars) {

		#
		# TODO: Do error checking
		#

		$meta = array();
		foreach ($vars as $key => $value) {
			$meta[] = array(
				'key' => $key,
				'value' => $value,
				'serialized' => $key == 'support_packages' ? 1 : 0,
				'encrypted' => 0
			);
		}
		return $meta;
	}

	/**
	 * Edits the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being updated.
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 * @param array $vars An array of module info to update
	 * @return array A numerically indexed array of meta fields for the module row containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 */
	public function editModuleRow($module_row, array &$vars) {

		#
		# TODO: Do error checking
		#

		$meta = array();
		foreach ($vars as $key => $value) {
			$meta[] = array(
				'key' => $key,
				'value' => $value,
				'serialized' => $key == 'support_packages' ? 1 : 0,
				'encrypted' => 0
			);
		}
		return $meta;
	}

	/**
	 * Returns all fields to display to an admin attempting to edit a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */
	public function getAdminEditFields($package, $vars=null) {
		Loader::loadHelpers($this, array("Html"));

		$fields = new ModuleFields();

		// Create license key label
		$license_key = $fields->label(Language::_("LicenseModule.service_field.license_module_key", true), "license_module_key");
		// Create license key field and attach to label
		$license_key->attach($fields->fieldText("license_module_key", ($vars->license_module_key ?? null), array('id'=>"license_module_key")));
		// Set the label as a field
		$fields->setField($license_key);

		// Create license status label
		$label = $fields->label(Language::_("LicenseModule.service_field.statuses", true));
		$license_status = $fields->label(Language::_("LicenseModule.service_field.statuses.reissue", true), "license_module_status");
		// Create license status field and attach the label
		$status = $fields->fieldCheckbox("license_module_status", "reissue", ($vars->license_module_status ?? null) == "reissue", array('id'=>"license_module_status"));
		$status->setLabel($license_status);
		$label->attach($status);
		// Set the label as a field
		$fields->setField($label);

		return $fields;
	}

	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * admin interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	public function getAdminServiceInfo($service, $package) {
		$row = $this->getModuleRow();

		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("admin_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);

		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));

		return $this->view->fetch();
	}

	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * client interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	public function getClientServiceInfo($service, $package) {
		$row = $this->getModuleRow();

		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("client_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);

		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));

		return $this->view->fetch();
	}

	/**
	 * Returns all tabs to display to an admin when managing a service whose
	 * package uses this module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @return array An array of tabs in the format of method => title. Example: array('methodName' => "Title", 'methodName2' => "Title2")
	 */
	public function getAdminTabs($package) {
		return array('tabLocations' => Language::_("LicenseModule.getAdminTabs.tabLocations", true));
	}

	/**
	 * Returns all tabs to display to a client when managing a service whose
	 * package uses this module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @return array An array of tabs in the format of method => title. Example: array('methodName' => "Title", 'methodName2' => "Title2")
	 */
	public function getClientTabs($package) {
		return array('tabManageLicense' => array('name' => Language::_("LicenseModule.getClientTabs.tabManageLicense", true)));
	}

	/**
	 * Set location details (IPs/Domains/Paths) tab
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $get Any GET parameters
	 * @param array $post Any POST parameters
	 * @param array $files Any FILES parameters
	 * @return string The string representing the contents of this tab
	 */
	public function tabLocations($package, $service, array $get=null, array $post=null, array $files=null) {
		$this->view = new View("tab_locations", "default");
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$vars = $this->serviceFieldsToObject($service->fields);

		$vars->license_module_ips = implode("\n", $vars->license_module_ips);
		$vars->license_module_domains = implode("\n", $vars->license_module_domains);
		$vars->license_module_paths = implode("\n", $vars->license_module_paths);

		// Save configurable options
		if (!empty($post)) {
			$data = array();
			$data['ip'] = explode("\n", $post['license_module_ips']);
			foreach ($data['ip'] as &$ip) {
				$ip = trim($ip);
			}
			unset($ip);

			$data['domain'] = explode("\n", $post['license_module_domains']);
			foreach ($data['domain'] as &$domain) {
				$domain = trim($domain);
			}
			unset($domain);

			$data['path'] = explode("\n", $post['license_module_paths']);
			foreach ($data['path'] as &$path) {
				$path = trim($path);
			}
			unset($path);

			$this->updateLicenseData($service, $data, true);
			$vars = (object)$post;
		}

		$this->view->set("vars", $vars);

		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		return $this->view->fetch();
	}

	/**
	 * Manage License tab
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $get Any GET parameters
	 * @param array $post Any POST parameters
	 * @param array $files Any FILES parameters
	 * @return string The string representing the contents of this tab
	 */
	public function tabManageLicense($package, $service, array $get=null, array $post=null, array $files=null) {
		$row = $this->getModuleRow();
		$this->view = new View("tab_manage_license", "default");
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$vars = $this->serviceFieldsToObject($service->fields);

		// Save configurable options
		if (!empty($post)) {
			$vars = (object)$post;

			if (!isset($post['license_module_status']))
				$post['license_module_status'] = "";

			if (isset($row->meta->allow_reissue) && $row->meta->allow_reissue === "true") {
				$this->updateLicenseData($service, array('status' => $post['license_module_status']));
			}
		}

		$this->view->set("vars", $vars);
		$this->view->set("row", $row);

		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		return $this->view->fetch();
	}

	/**
	 * Fetches the service containing the given license key
	 *
	 * @param int $module_id The ID of the module to fetch on
	 * @param string $license_key The license key to search services for
	 * @return mixed A stdClass object representing the service, false otherwise
	 */
	public function getLicense($module_id, $license_key) {
		Loader::loadModels($this, array("Services"));
		$services = $this->Services->searchServiceFields($module_id, "license_module_key", $license_key);

		if (empty($services))
			return false;
		return $services[0];
	}

	/**
	 * Returns license data for the given service
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param boolean $allow_wildcard True to allow wildcard values if the module settings permit, false to not include them
	 * @return array An array of license data including:
	 * 	- domain An array of domains set for the license
	 * 	- ip An array of IP addresses set for the license
	 * 	- path An array of install paths for the license
	 * 	- status The service status (active, canceled, pending, suspended)
	 * 	- version The version recorded for this service
	 * 	- allow_reissue "true" if the license can be reissued, false otherwise
	 * 	- label The label used to ID this license package
	 * 	- custom
	 * 	- addons
	 * 	- updates Date that support and updates are good through, false if no support and updates, null if never expire
	 */
	public function getLicenseData(stdClass $service, $allow_wildcard = true) {
		$fields = $this->serviceFieldsToObject($service->fields);
		$row = $this->getModuleRow();

		// Determine the license type
		Configure::load("license_manager", PLUGINDIR . "license_manager" . DS . "config" . DS);
		$custom = array(
			'license_type' => (isset($service->package_pricing->period) ? $service->package_pricing->period : ""),
			'cancellation_date' => $service->date_canceled
		);

		// Determine if this client matches a trial license
		if (Configure::get("LicenseManager.trial_client_id") == $service->client_id)
			$custom['license_type'] = "trial";

		$updates = false;
		$addons = null;

		// Set included support and updates
		if ($row->meta->support_term != "") {
			// If service was canceled, support ends when canceled
			if ($service->date_canceled)
				$updates = date("c", strtotime($service->date_canceled . "Z"));
			// Onetime support good for forever
			elseif ($row->meta->support_period == "onetime")
				$updates = null;
			// Calculate end of gratis period (date added + included period)
			else
				$updates = date("c", strtotime($service->date_added . "Z +" . (int)($row->meta->support_term) . " " . $row->meta->support_period));
		}

		if (!isset($this->Services))
			Loader::loadModels($this, array("Services"));

		if (!isset($this->ModuleManager))
			Loader::loadModels($this, array("ModuleManager"));

        // Check for support and updates addon services
		foreach ($this->Services->getAllChildren($service->id, "active") as $addon) {
			if (!$addons)
				$addons = array();

			// Support and updates enabled, set date good through
			if (
                (!empty($row->meta->support_packages)
                    && in_array($addon->package->id, $row->meta->support_packages)
                    && $addon->status == "active"
                )
            ) {
                $temp_date = null;
				if ($addon->date_canceled) {
					$temp_date = date("c", strtotime($addon->date_canceled . "Z"));
                } elseif ($addon->date_renews) {
					$temp_date = date("c", strtotime($addon->date_renews . "Z"));
                } else {
                    // Onetime support good for forever
                    $updates = null;
                    break;
                }

                // Use the farthest forward date
                if (strtotime($temp_date) > strtotime($updates)) {
                    $updates = $temp_date;
                }
			}

			$addon_fields = array();
			foreach ($addon->fields as $addon_field) {
				if ($addon_field->encrypted == "0")
					$addon_fields[$addon_field->key] = $addon_field->value;
			}

			$module = $this->ModuleManager->get($addon->package->module_id, false, false);

			$addons[] = array(
				'qty' => isset($addon->qty) ? $addon->qty : null,
				'fields' => $addon_fields,
				'module' => isset($module->class) ? $module->class : null,
				'package_id' => $addon->package->id
			);
		}

		// Set wildcard for domain, ip, and path if not set to validate
		$domain = isset($fields->license_module_domains) ? $fields->license_module_domains : array();
		$ip = isset($fields->license_module_ips) ? $fields->license_module_ips : array();
		$path = isset($fields->license_module_paths) ? $fields->license_module_paths : array();
		$wildcard = "*";
		if ($allow_wildcard) {
			if ((!isset($row->meta->validate['domain']) || $row->meta->validate['domain'] != "true") && !in_array($wildcard, $domain))
				$domain[] = $wildcard;
			if ((!isset($row->meta->validate['ip']) || $row->meta->validate['ip'] != "true") && !in_array($wildcard, $ip))
				$ip[] = $wildcard;
			if ((!isset($row->meta->validate['path']) || $row->meta->validate['path'] != "true") && !in_array($wildcard, $path))
				$path[] = $wildcard;
		}

		return array(
			'domain' => $domain,
			'ip' => $ip,
			'path' => $path,
			'status' => isset($service->status) ? $service->status : null,
			'version' => isset($fields->license_module_version) ? $fields->license_module_version : null,
			'allow_reissue' => isset($row->meta->allow_reissue) ? $row->meta->allow_reissue : "false",
			'label' => isset($row->meta->label) ? $row->meta->label : null,
			'custom' => $custom,
			'addons' => $addons,
			'updates' => $updates
		);
	}

	/**
	 * Updates the license data for the service using the given fields
	 *
	 * @param stdClass A stdClass object representing the service
	 * @param array $vars An array of key/value pairs where each key is the field and each value is the value to save for that field
	 * @param boolean $override Set to true if overriding the details (no verification on domain, ip, or path and overwrites existing values as opposed to appending)
	 */
	public function updateLicenseData(stdClass $service, $vars, $override = false) {
		$license = $this->getLicenseData($service, false);
		$row = $this->getModuleRow();

		Loader::loadModels($this, array("Services"));
		$fields = array(
			'callhome' => "license_module_callhome",
			'version' => "license_module_version",
			'status' => "license_module_status",
			'domain' => "license_module_domains",
			'ip' => "license_module_ips",
			'path' => "license_module_paths"
		);

		$instances = $row->meta->instances;

		// Increment installed instances provided by instance package
		if (isset($row->meta->instance_package)) {
			foreach ((array)$license['addons'] as $addon) {
				if ($addon['package_id'] == $row->meta->instance_package) {
					$instances += $addon['qty'];
					// Continue because there may be more services of this package
				}
			}
		}

		foreach ($fields as $key => $name) {
			if (array_key_exists($key, $vars)) {

				if ($key == "domain" || $key == "ip" || $key == "path") {
					if (!is_array($license[$key]) || $override)
						$license[$key] = array();

					if (is_array($vars[$key])) {
						foreach ($vars[$key] as $element) {
							$existing_count = $key === "ip"
								? $this->ipCounts($this->getIpVersion($element), $license[$key])
								: count($license[$key]);
							if ($override || ($existing_count < $instances && !in_array($element, $license[$key])))
								$license[$key][] = $element;
						}
					}
					else {
						$existing_count = $key === "ip"
							? $this->ipCounts($this->getIpVersion($vars[$key]), $license[$key])
							: count($license[$key]);
						if ($override || ($existing_count < $instances && !in_array($vars[$key], $license[$key])))
							$license[$key][] = $vars[$key];
					}
				}
				else
					$license[$key] = $vars[$key];

				$this->Services->editField($service->id, array('key' => $name, 'value' => $license[$key]));
			}
		}
	}

	/**
	 * Detects the IP version from the given IP address
	 *
	 * @param string $ip
	 * @return string The IP version (v4 or v6)
	 */
	private function getIpVersion($ip) {
		return strpos($ip, ":") !== false
			? "v6"
			: "v4";
	}

	/**
	 * Counts the number of IP addresses of the given type in the array of IPs
	 *
	 * @param string $version The IP version (v4, or v6)
	 * @param array $ips An array of v4 and v6 IP addresses
	 * @return int The number of v4 or v6 IP address found
	 */
	private function ipCounts($version, array $ips) {
		$count = 0;
		foreach ($ips as $ip) {
			if ($version === $this->getIpVersion($ip)) {
				$count++;
			}
		}
		return $count;
	}

	/**
	 * Validates that a given license key is not already in use
	 *
	 * @param string $license_key The license key to validate
	 * @param int $module_id The ID of the module to search on
	 * @param int $service_id The ID of the service to allow duplicate on
	 * @return boolean True if it is valid, false otherwise
	 */
	public function validateLicenseKeyUnique($license_key, $module_id, $service_id = null) {
		$license = $this->getLicense($module_id, $license_key);
		if (!$license || ($license && $license->id == $service_id))
			return true;
		return false;
	}

	/**
	 * Returns all license methods supported
	 *
	 * @return array An array of key/value pairs
	 */
	private function licenseMethods() {
		return array(
			'default' => Language::_("LicenseModule.license_methods.default", true)
		);
	}

	/**
	 * Returns all license key methods supported
	 *
	 * @return array An array of key/value pairs
	 */
	private function licenseKeyMethods() {
		return array(
			'auto' => Language::_("LicenseModule.key_methods.auto", true)
		);
	}
}
?>