<?php
/**
 * License Keys send emails controller
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys
 * @copyright Copyright (c) 2020, We Hear You 2, Inc.
 */

require_once PLUGINDIR . 'license_keys' . DS . 'vendors' . DS . 'reachmail'. DS .'rmapi.class.php';

class AdminSendEmails extends LicenseKeysController {

    /**
     * @var array Accepted email options
     */
    private $email_fields = [
        'from_name',
        'from_address',
        'subject',
        'html',
        'product_name',
        'from',
        'to'
    ];

    public function preAction() {
        parent::preAction();
        $this->structure->setDefaultView( APPDIR );
        $this->structure->setView( null, $this->orig_structure_view );

        $this->requireLogin();
        // Filters from the first step must be defined
        $session = $this->read();
        $this->uses( [ 'LicenseKeys.LicenseKeysList', 'Contacts' ] );
        Language::loadLang( 'admin_send_emails', null, PLUGINDIR . 'license_keys' . DS . 'language' . DS );
        Language::loadLang( 'admin_send_emails_preview', null, PLUGINDIR . 'license_keys' . DS . 'language' . DS );
    }

    /**
     * List mailings
     */
    public function index() {
        if (!empty($this->post)) {
            // Only set the expected fields
            $data = array_intersect_key( $this->post, array_flip( $this->email_fields ) );
            $this->uses( [ 'LicenseKeys.LicenseKeysList' ] );

            // Email content valid?
            $this->LicenseKeysList->validate( $data );
            if ( ( $errors = $this->LicenseKeysList->errors() ) ) {
                $this->setMessage( 'error', $errors, false, null, false );
                $vars = (object) $this->post;
            } else {
                // Email passed validation, preview the email
                $data['from_name']='AbanteCartPlus';
                $data['from_address'] = 'notifications@abantecartplus.com';
                $this->write('email',$data);
                $this->write( 'extension_name', $data['product_name'] );
                $this->write('from',$data['from']);
                $this->write('to',$data['to']);
                $this->redirect( $this->base_uri . 'plugin/license_keys/admin_send_emails/preview/' );
            }
        }

        // Set the email information from the session if available
        if (!isset($vars)) {
            $session = $this->read();
            if (is_array($session) && array_key_exists('email', $session)) {
                $vars = (object)$session['email'];
            }
        }

        $this->set('vars', (isset($vars) ? $vars : $this->getDefaultVars()));

        $this->setWysiwyg();
    }

    /**
     * Preview the email
     */
    public function preview() {
        // The email from the previous step must be defined
        $session = $this->read();
        if ( ! is_array( $session ) || ! array_key_exists( 'email', $session ) ) {
            $this->redirect( $this->base_uri . 'plugin/license_keys/admin_send_emails/' );
        }

        $contacts = $this->LicenseKeysList->getEmailsByExtensionName( $session['extension_name'],$session['from'],$session['to'] );
        // Create the mailing job
        if (!empty($this->post)) {
            $data['subject'] = $session['email']['subject'];
            //$data['from_name'] = $session['email']['from_name'];
            //$data['from_address'] = $session['email']['from_address'];
            $data['from_name'] = 'AbanteCartPlus';
            $data['from_address'] = 'notifications@abantecartplus.com';
            $data['html'] = $session['email']['html'];
            $data['contacts'] = json_decode(json_encode($contacts),true);
            $response = $this->send($data);
            if ($response['http_status'] === 200) {
                $this->clear();
                $this->flashMessage('message', Language::_('AdminSendEmails.send.success', true));
                $this->redirect( $this->base_uri . 'plugin/license_keys/admin_send_emails/' );
            } else {
                if ( $response["service_response"]->Failures ) {
                    $this->clear();
                    $error_message = "";
                    if ( sizeof( $response["service_response"]->FailedAddresses ) > 0 ) {
                        $failed_addresses = $response["service_response"]->FailedAddresses;
                        foreach ( $failed_addresses as $failed ) {
                            $error_message .= Language::_( 'AdminSendEmails.send.failed', true ) . $failed->Email . '.' . Language::_( 'AdminSendEmails.send.reason', true ) . $failed->Reason . "<br>";
                        }
                    } elseif ( $response["http_status_text"] ) {
                        $error_message = $response->http_status_text;
                    }
                    $this->flashMessage('message', html_entity_decode( $error_message ), true);
                    //echo $error_message;
                    $this->redirect( $this->base_uri . 'plugin/license_keys/admin_send_emails/' );
                }
            }
        }

        $this->uses( [ 'LicenseKeys.LicenseKeysList' ] );

        // Fetch the total number of email recipients
        $total = $this->LicenseKeysList->getEmailTotals( $session['extension_name'],$session['from'],$session['to']  );

        // Generate a sample email using one of the recipients
        $sample = (object) [];
        if ( $total > 0 ) {
            // Attempt to generate a sample email for review
            try {
                $sample = (object) array_merge(
                    (array) $session['email'],
                    (array) $this->LicenseKeysList->getSample( $session['email'] )
                );
            } catch ( H2o_Error $e ) {
                // Parse error, redirect back to edit the email template again
                $this->flashMessage(
                    'error',
                    Language::_( 'AdminSendEmails.!error.parse_error', true, $e->getMessage() ),
                    null,
                    false
                );
                $this->redirect( $this->base_uri . 'plugin/license_keys/admin_send_emails/' );
            }

            // Write the HTML sample to the session to be fetched by the iframe
            $this->write( 'sample_html', $sample->html );
        }

        $this->set( compact( 'total', 'sample' ) );
    }

    /**
     * Sample HTML email page
     */
    public function sample() {
        $html = '';
        $session = $this->read();

        if (isset($session['sample_html'])) {
            $html = $session['sample_html'];
        }

        echo $this->partial('partials' . DS . 'sample_email', compact('html'));
        exit();
    }

    /**
     * Returns default values for composing an email
     *
     * @return stdClass An stdClass object representing input fields with default values set
     */
    protected function getDefaultVars() 	{
        $vars = new stdClass();

        // Set the email from name/email to the current staff member's name/email
        if (($staff = $this->getStaff())) {
            $vars->from_name = $staff->first_name . ' ' . $staff->last_name;
            $vars->from_address = $staff->email;
        }

        return $vars;
    }

    private function send($data) {
        $rmapi = new RMAPI('8vrFiIReclqFV6RxxY2XmEce97r4qbkEtkknIiSB0GrjGVu-B2ESFFKtXARFOow2');
        $recipients=array();
        // Retrieve account information array
        $account_info = $rmapi->rm_administrationUsersCurrent();
        $AccountId = $account_info['service_response']->AccountId;
        foreach ($data['contacts'] as $contact) {
            $recipients[]=array("To" => array(
                "Name"    => $contact['first_name'].' '.$contact['last_name'],
                "Email"   => $contact['email']),
                "Subject" =>  $data['subject'],
                "Substitutions" => array('name' => $contact['first_name'].' '.$contact['last_name'])
            );
        }

        $from = array(
            "Name" => 'AbanteCartPlus',
            "Email" => 'notifications@abantecartplus.com'
        );
        $to = array(
            "Name" => 'AbanteCartPlus',
            "Email" => 'replies@abantecartplus.com'
        );
        $request = array (
            "Personalizations"   =>  $recipients,
            "From" => $from,
            "ReplyTo" => $to,
            "Contents" => array(
                array(
                    "Type" => "text/html; charset=UTF-8",
                    "Content" =>  trim($data["html"])."<br>This message was sent by We Hear You 2, Inc. / AbanteCart Plus division<br>
                                                       We Hear You 2, Inc | 570 Hwy 200 | Montevallo, AL 35115",
                )),
            "Settings"      => array("Tracking"=>true)
        );

        return $rmapi->rm_easySmtpAdvancedDelivery($AccountId, $request);
    }

}