<?php
/**
 * License Manager licnese validation controller
 *
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class Validate extends LicenseManagerController {

	/**
	 * @var string The delimiter used to separated the data from the signature in a response
	 */
	const RESPONSE_DELIMITER = '$';
	/**
	 * @var string The ID of the log entry
	 */
	private $log_id;
	/**
	 * @var array An array of settings
	 */
	private $settings;

	public function preAction() {
		// Disable CSRF for requests
		Configure::set("Blesta.verify_csrf_token", false);

		parent::preAction();

		$this->uses(array("LicenseManager.LicenseManagerKeys", "LicenseManager.LicenseManagerLicenses", "LicenseManager.LicenseManagerSettings"));

		$this->settings = $this->LicenseManagerSettings->getSettings($this->company_id);

		$this->log("input", serialize($this->post));
	}
	/**
	 * Validates an incoming license request, replies with license data if the
	 * license is valid, or no reply if the license key does not exist or is
	 * not given.
	 *
	 * Expects The following POST fields:
	 * 	- key The license key to validate
	 * 	- data The license info + signature to verify this is a trusted request. License info includes:
	 * 		- domain The domain the installation is running on
	 * 		- ip The IP address the installation is running on
	 * 		- path The installation path
	 * 		- ... misc other fields
	 */
	public function index() {
		$keys = false;
		if (isset($this->post['key']))
			$keys = $this->LicenseManagerKeys->getKeys($this->post['key']);

		if ($keys) {
			if (isset($this->post['data']) && $this->verifySignature($this->post['data'], $keys, true)) {
                // Decrypt and sanitize the license data through JSON
                // to avoid PHP Object Injection with unserialize()
				$this->post['data'] = json_decode(
                    json_encode(
                        unserialize($this->decryptData($this->post['data'], $keys))
                    ),
                    true
                );

				// Handle additional instance requests
				$this->post['data']['callhome'] = date("c");
				$this->LicenseManagerLicenses->updateLicenseData($this->post['key'], $this->post['data']);

				if ($this->LicenseManagerLicenses->verify($this->post['key'], $this->post['data'])) {

					// Handle reissue updates
					$this->LicenseManagerLicenses->updateLicenseData($this->post['key'], $this->post['data']);

					$data = $this->LicenseManagerLicenses->getClientLicenseData($this->post['key']);

					if ($data)
						$data = array('time' => time()) + (array)$data;

					$this->sendEncryptedResponse($data, $keys->aes_key, $keys->iv, $keys->private_key, $keys->hmac_key);
				}
			}
		}

		return false;
	}

	/**
	 * Sends the RSA public key
	 *
	 * Expects The following POST fields:
	 * 	- key The license key to fetch the encryption key for
	 * 	- data The license info + signature to verify this is a trusted request. license info includes:
	 * 		- domain The domain the installation is running on
	 * 		- ip The IP address the installation is running on
	 * 		- path The installation path
	 * 		- ... misc other fields
	 */
	public function keyexchange() {

		$keys = false;
		if (isset($this->post['key']))
			$keys = $this->LicenseManagerKeys->getKeys($this->post['key']);

		// Verify data can be trusted
		if ($keys && isset($this->post['data']) && $this->verifySignature($this->post['data'], $keys, false)) {

			#
			# TODO: base64_decode(unserialize($this->post['data'])) to update/add to the DB
            # - be sure to json_encode/decode the unserialized data to avoid PHP Object Injection
			#

			$this->sendPlainResponse($keys->public_key, $keys->hmac_key);
		}

		return false;
	}

	/**
	 * Verifies that the data given can be trusted
	 *
	 * @param string $data The complete data set, including the attached signature
	 * @param stdClass $keys A stdClass object containing the keys for the license:
	 * 	- private_key
	 * 	- hmac_key
	 * @param boolean $encrypted True if the data is encrypted, false otherwise
	 * @return boolean True if the data can be trusted, false otherwise
	 */
	private function verifySignature($data, $keys, $encrypted=true) {
		$parts = explode(self::RESPONSE_DELIMITER, $data);

		if (count($parts) == 2) {
			// If encrypted, data is signed using an RSA public key
			if ($encrypted)
				return $this->LicenseManagerKeys->verifyRsa($parts[0], $keys->private_key, $parts[1], $keys->hmac_key);
			// If not encrypted, data is signed using HMAC
			else
				return $this->LicenseManagerKeys->verifyHmac($parts[0], $keys->hmac_key, $parts[1]);
		}
		return false;
	}

	/**
	 * Decrypts the given encrypted data using the given key data
	 *
	 * @param string $data The encrypted data + signature
	 * @param stdClass $keys A stdClass object containing the keys for this data:
	 * 	- aes_key
	 * 	- iv
	 * @return string The decrypted data
	 */
	private function decryptData($data, $keys) {
		$parts = explode(self::RESPONSE_DELIMITER, $data);

		if (count($parts) == 2)
			return $this->LicenseManagerKeys->decrypt($parts[0], $keys->aes_key, $keys->iv);
		return null;
	}

	/**
	 * Outputs the HMAC signed plain-text response
	 *
	 * @param string $data The data to send
	 * @param string $shared_secret The shared secret value
	 */
	private function sendPlainResponse($data, $shared_secret) {
		$response = $this->getPlainResponse($data, $shared_secret);
		$this->log("output", $response);
		echo $response;
	}

	/**
	 * Generates the signed plain-text response
	 *
	 * @param string $data The data to send
	 * @param string $shared_secret The shared secret value
	 * @return string The HMAC signed plain-text response
	 */
	private function getPlainResponse($data, $shared_secret) {
		return $data . self::RESPONSE_DELIMITER . $this->LicenseManagerKeys->signHmac($data, $shared_secret);
	}

	/**
	 * Outputs the RSA signed encrypted response
	 *
	 * @param mixed $data The data to send
	 * @param string $aes_key The AES key to use to encrypt the data
	 * @param string $iv The IV to use to encrypt the data
	 * @param string $private_key The RSA private key to use to sign the encrypted data
	 * @param string $shared_secret The shared secret value
	 */
	private function sendEncryptedResponse($data, $aes_key, $iv, $private_key, $shared_secret=null) {
		$response = $this->getEncryptedResponse($data, $aes_key, $iv, $private_key, $shared_secret);
		$this->log("output", $response);
		echo $response;
	}

	/**
	 * Generates the RSA signed encrypted response
	 *
	 * @param mixed $data The data to send
	 * @param string $aes_key The AES key to use to encrypt the data
	 * @param string $iv The IV to use to encrypt the data
	 * @param string $private_key The RSA private key to use to sign the encrypted data
	 * @param string $shared_secret The shared secret value
	 * @return string The signed encrypted responses
	 */
	private function getEncryptedResponse($data, $aes_key, $iv, $private_key, $shared_secret=null) {
		$data = serialize($data);

		$y = $this->LicenseManagerKeys->encrypt($data, $aes_key, $iv);
		$sig = $this->LicenseManagerKeys->signRsa($y, $private_key, $shared_secret);
		return $y . self::RESPONSE_DELIMITER . $sig;
	}

	/**
	 * Logs data for the given direction with the given data
	 *
	 * @param string $direction input or output
	 * @param strign $data The data to log
	 */
	private function log($direction, $data = null) {
		if ($this->log_id == null)
			$this->log_id = uniqid();

		// Log requests
		if (!empty($this->settings['log_file'])) {
			try {
				file_put_contents($this->settings['log_file'],
					sprintf($this->settings['log_format'],
						$this->log_id,
						$direction,
						date("c"),
						isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : null,
						isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : null,
						$data
					),
					FILE_APPEND
				);
			}
			catch (Exception $e) {
				// Unable to write to the file
			}
		}
	}
}