<?php
/**
 * License Manager plugin handler
 * 
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseManagerPlugin extends Plugin {
	
	public function __construct() {
		// Load components required by this plugin
		Loader::loadComponents($this, array("Input"));
		
		Language::loadLang("license_manager_plugin", null, dirname(__FILE__) . DS . "language" . DS);
		
		$this->loadConfig(dirname(__FILE__) . DS . "config.json");
	}

	/**
	 * Performs any necessary bootstraping actions
	 *
	 * @param int $plugin_id The ID of the plugin being installed
	 */
	public function install($plugin_id) {
		$this->setDefaultSettings();
	}
	
	/**
	 * Performs migration of data from $current_version (the current installed version)
	 * to the given file set version
	 *
	 * @param string $current_version The current installed version of this plugin
	 * @param int $plugin_id The ID of the plugin being upgraded
	 */
	public function upgrade($current_version, $plugin_id) {
		// Upgrade if possible
		if (version_compare($this->getVersion(), $current_version, ">")) {
			// Handle the upgrade, set errors using $this->Input->setErrors() if any errors encountered
			
			if (version_compare("1.2.0", $current_version, "<=")) {
				$this->setDefaultSettings();
			}
		}
	}
	
	/**
	 * Performs any necessary cleanup actions
	 *
	 * @param int $plugin_id The ID of the plugin being uninstalled
	 * @param boolean $last_instance True if $plugin_id is the last instance across all companies for this plugin, false otherwise
	 */
	public function uninstall($plugin_id, $last_instance) {
		if (!isset($this->Record))
			Loader::loadComponents($this, array("Record"));
			
		if ($last_instance) {
			// Remove settings created by this plugin
			$this->Record->from("company_settings")->like("key", "license_manager.%")->delete();
		}
	}
	
	/**
	 * Sets default settings for the plugin
	 */
	private function setDefaultSettings() {
		if (!isset($this->LicenseManagerSettings))
			Loader::loadModels($this, array("LicenseManager.LicenseManagerSettings"));
			
		$root_dir = rtrim(realpath(ROOTWEBDIR . ".." . DS), DS) . DS;
		$settings = array(
			'log_file' => $root_dir . "license_log.txt",
			'log_trial_file' => $root_dir . "license_trial_log.txt",
			'log_format' => "%1\$s\t%2\$s\t%3\$s\t%4\$s\t%5\$s\t%6\$s\n"
		);
		$this->LicenseManagerSettings->setSettings(Configure::get("Blesta.company_id"), $settings);
	}
	
	/**
	 * Returns all events to be registered for this plugin (invoked after install() or upgrade(), overwrites all existing events)
	 *
	 * @return array A numerically indexed array containing:
	 * 	- event The event to register for
	 * 	- callback A string or array representing a callback function or class/method. If a user (e.g. non-native PHP) function or class/method, the plugin must automatically define it when the plugin is loaded. To invoke an instance methods pass "this" instead of the class name as the 1st callback element.
	 */	
	public function getEvents() {
		return array(
			array(
				'event' => "Services.add",
				'callback' => array("this", "handleService")
			),
			array(
				'event' => "Services.edit",
				'callback' => array("this", "handleService")
			),
		);
	}
	
	/**
	 * Handle the service event
	 */
	public function handleService($event) {
		try {
			$params = $event->getParams();
			$statuses = array("pending", "in_review");
			
			if (isset($params['service_id']) && isset($params['vars']['status']) && in_array($params['vars']['status'], $statuses)) {
				Loader::loadComponents($this, array("Record"));
				
				// Verify service is support package for a license
				$fields = array(
					"services.date_renews", "services.parent_service_id",
					'service_fields.value' => "license_key", "pricings.term",
					"pricings.period"
				);
				$support_service = $this->Record->select($fields)->from("services")->
					on("service_fields.service_id", "=", "services.parent_service_id", false)->
					innerJoin("service_fields", "service_fields.key", "=", "license_module_key")->
					innerJoin("package_pricing", "package_pricing.id", "=", "services.pricing_id", false)->
					innerJoin("pricings", "pricings.id", "=", "package_pricing.pricing_id", false)->
					innerJoin("packages", "packages.id", "=", "package_pricing.package_id", false)->
					on("module_row_meta.value", "=", "packages.id", false)->
					innerJoin("module_row_meta", "module_row_meta.key", "=", "support_package")->
					innerJoin("module_rows", "module_rows.id", "=", "module_row_meta.module_row_id", false)->
					innerJoin("modules", "modules.id", "=", "module_rows.module_id", false)->
					where("services.id", "=", $params['service_id'])->
					where("services.date_renews", "!=", null)->
					where("services.parent_service_id", "!=", null)->fetch();
				
				if ($support_service) {
					if (!isset($this->LicenseManagerLicenses))
						Loader::loadModels($this, array("LicenseManager.LicenseManagerLicenses"));
						
					$data = $this->LicenseManagerLicenses->getClientLicenseData($support_service->license_key);
					
					// Offset from current updates date if updates are good past now
					if ($data['updates'] > date("c")) {
						$date_renews = $this->LicenseManagerLicenses->dateToUtc(date("c", strtotime($data['updates'] . "Z +" . $support_service->term . " " . $support_service->period)));
						
						$this->Record->where("services.id", "=", $params['service_id'])->
							update("services", array('date_renews' => $date_renews));
					}
				}
			}
		}
		catch (Exception $e) {
			// Nothing to do
		}
	}
}
?>