<?php
/**
 * License Manager Keys
 * 
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseManagerKeys extends LicenseManagerModel {
	
	public function __construct() {
		parent::__construct();
		
	}
	
	/**
	 * Retrieves RSA encryption keys, and the shared secret key for the given license key.
	 * If the public/private key pair does not exists it is generated.
	 *
	 * @param string $licnese_key The license key to fetch encryption keys for
	 * @return mixed A stdClass object containing encrpytion keys, false if no such license exists. Keys include:
	 * 	- private_key The RSA private key
	 * 	- public_key The RSA public key
	 * 	- hmac_key The HMAC key (also known as the "shared secret key")
	 * 	- aes_key The AES encryption key (HMAC(license_key, hmac_key, sha256))
	 * 	- iv The initialization vector for AES encryption (HMAC(public_key, hmac_key, sha256))
	 */
	public function getKeys($license_key) {
		
		$service_info = $this->getServiceInfo($license_key);
		
		$keys = false;
		if ($service_info) {
			$keys = new stdClass();
			
			$service = $this->Services->get($service_info->id);
			
			// hmac_key (e.g. shared secret) comes from the module row
			$shared_secret = $this->ModuleManager->getRowMeta($service_info->module_row_id, "shared_secret");
			if ($shared_secret)
				$shared_secret = $shared_secret->shared_secret;

			$keys->hmac_key = $shared_secret;
			
			foreach ($service->fields as $i => $field) {
				
				switch ($field->key) {
					case "private_key":
					case "public_key":
						$keys->{$field->key} = $field->value;
						break;
				}
			}
			
			if (!isset($keys->private_key)) {
				$key_pair = $this->generateKeyPair(Configure::get("LicenseManager.rsa_key_size"));
				$keys->private_key = $key_pair['privatekey'];
				$keys->public_key = $key_pair['publickey'];
				
				// Save the key pair
				$this->Services->editField($service->id, array('key' => "private_key", 'value' => $keys->private_key));
				$this->Services->editField($service->id, array('key' => "public_key", 'value' => $keys->public_key));
				
			}
			
			if (isset($keys->public_key)) {
				$keys->aes_key = $this->hash($license_key, "sha256", $keys->hmac_key);
				$keys->iv = $this->hash($keys->public_key, "sha256", $keys->hmac_key);
			}
		}
		
		return $keys;
	}
	
	/**
	 * Generates a public/private RSA key pair
	 *
	 * @param int $bits The length of the RSA key in bits (1024, 2048, 3072, 4096, etc.)
	 * @return array
	 */
	public function generateKeyPair($bits=1024) {
		$this->loadCrypto(array("RSA"));
		
		$key_pair = $this->Crypt_RSA->createKey($bits);
		
		unset($this->Crypt_RSA);
		return $key_pair;
	}
	
	/**
	 * Signs data using an HMAC
	 *
	 * @param string $data The data to sign
	 * @param string $hmac_key The key used to sign the data using HMAC
	 * @param string $hash The hash to use (md5, sha1, sha256, sha512, etc.)
	 * @return string The signed data in base64 format
	 */
	public function signHmac($data, $hmac_key, $hash="sha256") {
		return base64_encode($this->hash($data, $hash, $hmac_key));
	}
	
	/**
	 * Verifies the signature using HMAC
	 *
	 * @param string $data The data to sign
	 * @param string $hmac_key The key used to sign the data using HMAC
	 * @param string $hash The hash to use (md5, sha1, sha256, sha512, etc.)
	 * @return boolean True if the signature matches, false otherwise
	 */
	public function verifyHmac($data, $hmac_key, $signature, $hash="sha256") {
		return $this->signHmac($data, $hmac_key, $hash) == $signature;
	}
	
	
	/**
	 * Signs the given data using RSA signature with the provided private key
	 *
	 * @param string $data Data to be signed. A hash of this data will automatically be computed and the hash signed.
	 * @param string $private_key The private key to use to sign the data
	 * @param string $hmac_key The key used to compute the HMAC hash, if null will only compute a normal hash of the data
	 * @param string $hash The hash to use (md5, sha1, sha256, sha512, etc.)
	 * @return string The signature of the $data in base64 format
	 */
	public function signRsa($data, $private_key, $hmac_key=null, $hash="sha256") {
		$this->loadCrypto(array("RSA"));
		
		$this->Crypt_RSA->loadKey($private_key);
		
		// Generate a hash of $data, because computing signatures on large
		// strings is slow and unncessary
		$hash = $this->hash($data, $hash, $hmac_key);
		
		$signature = $this->Crypt_RSA->sign($hash);
		unset($this->Crypt_RSA);
		return base64_encode($signature);
	}
	
	/**
	 * Verifies the signature given for the data using the public key
	 *
	 * @param string $data The data to be verified. A hash of this data will automatically be computed and the hash verified.
	 * @param string $public_key The public key to use to verify the signature
	 * @param string $signature A base64 encoded signature to be verified against
	 * @param string $hmac_key The key used to compute the HMAC hash, if null will only compute a normal hash of the data
	 * @param string $hash The hash to use (md5, sha1, sha256, sha512, etc.)
	 * @return boolean True if the signature is valid, false otherwise
	 */
	public function verifyRsa($data, $public_key, $signature, $hmac_key=null, $hash="sha256") {
		$this->loadCrypto(array("RSA"));

		// Generate a hash of $data, because computing signatures on large
		// strings is slow and unncessary
		$hash = $this->hash($data, $hash, $hmac_key);
		
		$this->Crypt_RSA->loadKey($public_key);
		
		$verified = $this->Crypt_RSA->verify($hash, base64_decode($signature));
		unset($this->Crypt_RSA);
		return $verified;
	}
	
	/**
	 * Performs AES-256 encryption on the given data using the given key
	 *
	 * @param string $data A string of data to encrypt
	 * @param string $key The key to use to encrypt the data
	 * @return string A base64 encoded string of encrypted data
	 */
	public function encrypt($data, $key, $iv) {
		$this->loadCrypto();

		// Set key
		$this->Crypt_AES->setKey($key);
		// Set IV
		$this->Crypt_AES->setIV($iv);

		return base64_encode($this->Crypt_AES->encrypt($data));
	}
	
	/**
	 * Perform AES-256 decryption on the givne data using the given key
	 *
	 * @param string $data A base64 encoded string of data to decrypt
	 * @param string $key The key to use to decrypt the data
	 * @return string The plain-text data
	 */
	public function decrypt($data, $key, $iv) {
		$this->loadCrypto();

		// Set key
		$this->Crypt_AES->setKey($key);
		// Set IV
		$this->Crypt_AES->setIV($iv);

		return $this->Crypt_AES->decrypt(base64_decode($data));
	}
	
	/**
	 * Computes a hash of the given data, optionally as an HMAC.
	 * 
	 * @param string $data The data to be hashed
	 * @param string $hash The hash to use (md5, sha1, sha256, sha512, etc.)
	 * @param string $hmac_key The key used to compute the HMAC hash, if null will only compute a normal hash of the data
	 * @return string A hash of the data in binary format
	 */
	private function hash($data, $hash, $hmac_key) {
		$this->loadCrypto();
		
		$this->Crypt_Hash->setHash($hash);
		$this->Crypt_Hash->setKey($hmac_key);
		return $this->Crypt_Hash->hash($data);
	}
}
?>