<?php
/**
 * License Manager Licenses
 *
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseManagerLicenses extends LicenseManagerModel {

	public function __construct() {
		parent::__construct();
	}

	/**
	 * Fetches all license data that is to be returned to the client
	 *
	 * @param string $license_key The license key to fetch license fields for
	 * @return array An array of license data to return to the client
	 */
	public function getClientLicenseData($license_key) {
		$service_info = $this->getServiceInfo($license_key);

		if (!$service_info)
			return null;

		$service = $this->Services->get($service_info->id);

		$module = $this->ModuleManager->initModule($service->package->module_id, Configure::get("Blesta.company_id"));
		$module->setModuleRow($module->getModuleRow($service->module_row_id));

		return $module->getLicenseData($service);
	}

	/**
	 * Fetches the license data stored for the given license key
	 *
	 * @param string The license key to fetch data for
	 * @return mixed A stdClass object containing all of the service fields for the service that matches the given license key, false if no such license key exists
	 */
	public function getLicenseData($license_key) {

		$service_info = $this->getServiceInfo($license_key);

		if ($service_info)
			return $this->serviceFieldsToObject($this->getServiceFields($service_info->id));

		return false;
	}

	/**
	 * Updates the license data with the given data
	 *
	 * @param string $license_key The license key to update services on
	 * @param array $vars A numerically indexed array of key
	 */
	public function updateLicenseData($license_key, array $vars) {

		$service_info = $this->getServiceInfo($license_key);

		if ($service_info) {
			$service = $this->Services->get($service_info->id);
			$module = $this->ModuleManager->initModule($service->package->module_id, Configure::get("Blesta.company_id"));
			$module->setModuleRow($module->getModuleRow($service->module_row_id));

			$module->updateLicenseData($service, $vars);
		}
	}

	/**
	 * Returns all module row meta data for the given module row
	 *
	 * @param int $module_row_id The module row to fetch module meta data for
	 * @return stdClass A stdClass object representing all of the module row meta fields
	 */
	public function getModuleRowData($module_row_id) {
		return $this->ModuleManager->getRowMeta($module_row_id);
	}

	/**
	 * Attempts to reissue the license to the given IP, host, and path
	 *
	 * @param string $license_key The license key to reissue
	 * @param string $ip The IP address of the application
	 * @param string $domain The Domain of the application
	 * @param string $path The installation path of the application
	 * @return mixed A stdClass object containing all of the service fields for the service that matches the given license key, false if no such license key exists
	 */
	private function reissue($license_key, $ip, $domain, $path) {

		$service_info = $this->getServiceInfo($license_key);

		if (!$service_info)
			return false;

		$this->Services->editField($service_info->id, array('key' => "license_module_status", 'value' => ""));
		$this->Services->editField($service_info->id, array('key' => "license_module_domains", 'value' => array($domain)));
		$this->Services->editField($service_info->id, array('key' => "license_module_ips", 'value' => array($ip)));
		$this->Services->editField($service_info->id, array('key' => "license_module_paths", 'value' => array($path)));

		return $this->serviceFieldsToObject($this->getServiceFields($service_info->id));
	}

	/**
	 * Verifies that the given license key is valid at the given location and
	 * updates the license data.
	 *
	 * @param string $license_key The license key to verify
	 * @param array $vars An array of license data to verify
	 * @return boolean True if the license data is valid
	 */
	public function verify($license_key, array $vars) {

		$service_info = $this->getServiceInfo($license_key);

		if (!$service_info)
			return false;

		$license_data = $this->serviceFieldsToObject($this->getServiceFields($service_info->id));

		if ($license_data) {

			// If license in reissue state, reissue
			if ($license_data->license_module_status == "reissue")
				$license_data = $this->reissue($license_key, $vars['ip'], $vars['domain'], $vars['path']);

			$module_row_meta = $this->getModuleRowData($service_info->module_row_id);

			// Check domain
			if (isset($module_row_meta->validate['domain']) && $module_row_meta->validate['domain'] == "true") {

				// Ensure case-insensitive check
				$vars['domain'] = strtolower($vars['domain']);

				// Strip "www." from beginning of domain if present
				if (substr($vars['domain'], 0, 4) == "www.")
					$vars['domain'] = substr($vars['domain'], 4);

				// Invalid domain?
				if ($vars['domain'] != "" && !in_array($vars['domain'], $license_data->license_module_domains))
					return false;
			}

			// Check IP
			if (isset($module_row_meta->validate['ip']) && $module_row_meta->validate['ip'] == "true") {

				// Invalid IP?
				if ($vars['ip'] != "" && !in_array($vars['ip'], $license_data->license_module_ips))
					return false;
			}

			// Check path
			if (isset($module_row_meta->validate['path']) && $module_row_meta->validate['path'] == "true") {

				// Invalid path?
				if (!in_array($vars['path'], $license_data->license_module_paths))
					return false;
			}

			return true;
		}
		return false;
	}
}
?>